@@ -1009,7 +1009,8 @@ bool Validate(int alg, bool thorough)
 	case 65: result = ValidateARIA(); break;
 	case 66: result = ValidateCamellia(); break;
 	case 67: result = ValidateWhirlpool(); break;
-	case 68: result = ValidateTTMAC(); break;
+	case 68: result = ValidateLSH(); break;
+	case 69: result = ValidateTTMAC(); break;
 	case 70: result = ValidateSalsa(); break;
 	case 71: result = ValidateChaCha(); break;
 	case 72: result = ValidateChaChaTLS(); break;
@@ -1059,12 +1060,13 @@ bool Validate(int alg, bool thorough)
 	case 9994: result = TestHuffmanCodes(); break;
 	// http://github.com/weidai11/cryptopp/issues/346
 	case 9993: result = TestASN1Parse(); break;
+	case 9992: result = TestASN1Functions(); break;
 	// http://github.com/weidai11/cryptopp/issues/242
-	case 9992: result = TestX25519(); break;
+	case 9991: result = TestX25519(); break;
 	// http://github.com/weidai11/cryptopp/issues/346
-	case 9991: result = TestEd25519(); break;
+	case 9990: result = TestEd25519(); break;
 # if defined(CRYPTOPP_ALTIVEC_AVAILABLE)
-	case 9990: result = TestAltivecOps(); break;
+	case 9989: result = TestAltivecOps(); break;
 # endif
 #endif
 
