@@ -73,6 +73,7 @@ bool ValidateAll(bool thorough)
 	pass=TestHuffmanCodes() && pass;
 	// http://github.com/weidai11/cryptopp/issues/346
 	pass=TestASN1Parse() && pass;
+	pass=TestASN1Functions() && pass;
 	// https://github.com/weidai11/cryptopp/pull/334
 	pass=TestStringSink() && pass;
 	// Always part of the self tests; call in Debug
@@ -106,6 +107,8 @@ bool ValidateAll(bool thorough)
 	pass=ValidateSHAKE() && pass;
 	pass=ValidateSHAKE_XOF() && pass;
 
+	pass=ValidateLSH() && pass;
+
 	pass=ValidateHashDRBG() && pass;
 	pass=ValidateHmacDRBG() && pass;
 
@@ -391,16 +394,16 @@ bool TestSettings()
 	bool hasAES = HasAES();
 	bool hasSHA1 = HasSHA1();
 	bool hasSHA2 = HasSHA2();
-	bool hasSHA512 = HasSHA512();
 	bool hasSHA3 = HasSHA3();
+	bool hasSHA512 = HasSHA512();
 	bool hasSM3 = HasSM3();
 	bool hasSM4 = HasSM4();
 
 	std::cout << "passed:  hasASIMD == 1";
 	std::cout << ", hasCRC32 == " << hasCRC32 << ", hasAES == " << hasAES;
 	std::cout << ", hasPMULL == " << hasPMULL << ", hasSHA1 == " << hasSHA1;
-	std::cout << ", hasSHA2 == " << hasSHA2 << ", hasSHA512 == " << hasSHA512;
-	std::cout << ", hasSHA3 == " << hasSHA3 << ", hasSM3 == " << hasSM3;
+	std::cout << ", hasSHA2 == " << hasSHA2 << ", hasSHA3 == " << hasSHA3;
+	std::cout << ", hasSHA512 == " << hasSHA512 << ", hasSM3 == " << hasSM3;
 	std::cout << ", hasSM4 == " << hasSM4 << "\n";
 # endif
 
@@ -409,8 +412,8 @@ bool TestSettings()
 	const bool hasPower7 = HasPower7();
 	const bool hasPower8 = HasPower8();
 	const bool hasPower9 = HasPower9();
-	const bool hasPMULL = HasPMULL();
 	const bool hasAES = HasAES();
+	const bool hasPMULL = HasPMULL();
 	const bool hasSHA256 = HasSHA256();
 	const bool hasSHA512 = HasSHA512();
 	const bool hasDARN = HasDARN();
@@ -418,7 +421,7 @@ bool TestSettings()
 	std::cout << "passed:  ";
 	std::cout << "hasAltivec == " << hasAltivec << ", hasPower7 == " << hasPower7;
 	std::cout << ", hasPower8 == " << hasPower8 << ", hasPower9 == " << hasPower9;
-	std::cout << ", hasPMULL == " << hasPMULL << ", hasAES == " << hasAES;
+	std::cout << ", hasAES == " << hasAES << ", hasPMULL == " << hasPMULL;
 	std::cout << ", hasSHA256 == " << hasSHA256 << ", hasSHA512 == " << hasSHA512;
 	std::cout << ", hasDARN == " << hasDARN << "\n";
 
@@ -711,7 +714,7 @@ bool TestMersenne()
 	bool pass = true;
 
 	try {rng.reset(new MT19937ar);}
-	catch (const PadlockRNG_Err &) {}
+	catch (const Exception &) {}
 
 	if(rng.get())
 	{
@@ -720,7 +723,7 @@ bool TestMersenne()
 
 	// Reset state
 	try {rng.reset(new MT19937ar);}
-	catch (const PadlockRNG_Err &) {}
+	catch (const Exception &) {}
 
 	if(rng.get())
 	{
@@ -761,6 +764,10 @@ bool TestPadlockRNG()
 		PadlockRNG& padlock = dynamic_cast<PadlockRNG&>(*rng.get());
 		pass = Test_RandomNumberGenerator(padlock);
 
+		// PadlockRNG does not accept entropy. However, the contract is no throw
+		const byte entropy[] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
+		(void)padlock.IncorporateEntropy(entropy, sizeof(entropy));
+
 		SecByteBlock zero(16), one(16), t(16);
 		std::memset(zero, 0x00, zero.size());
 		std::memset( one, 0xff,  one.size());
@@ -839,6 +846,10 @@ bool TestRDRAND()
 		RDRAND& rdrand = dynamic_cast<RDRAND&>(*rng.get());
 		pass = Test_RandomNumberGenerator(rdrand) && pass;
 
+		// RDRAND does not accept entropy. However, the contract is no throw
+		const byte entropy[] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
+		(void)rdrand.IncorporateEntropy(entropy, sizeof(entropy));
+
 		MaurerRandomnessTest maurer;
 		const unsigned int SIZE = 1024*10;
 		RandomNumberSource(rdrand, SIZE, true, new Redirector(maurer));
@@ -880,6 +891,10 @@ bool TestRDSEED()
 		RDSEED& rdseed = dynamic_cast<RDSEED&>(*rng.get());
 		pass = Test_RandomNumberGenerator(rdseed) && pass;
 
+		// RDSEED does not accept entropy. However, the contract is no throw
+		const byte entropy[] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
+		(void)rdseed.IncorporateEntropy(entropy, sizeof(entropy));
+
 		MaurerRandomnessTest maurer;
 		const unsigned int SIZE = 1024*10;
 		RandomNumberSource(rdseed, SIZE, true, new Redirector(maurer));
@@ -923,6 +938,10 @@ bool TestDARN()
 		DARN& darn = dynamic_cast<DARN&>(*rng.get());
 		pass = Test_RandomNumberGenerator(darn) && pass;
 
+		// DARN does not accept entropy. However, the contract is no throw
+		const byte entropy[] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
+		(void)darn.IncorporateEntropy(entropy, sizeof(entropy));
+
 		MaurerRandomnessTest maurer;
 		const unsigned int SIZE = 1024*10;
 		RandomNumberSource(darn, SIZE, true, new Redirector(maurer));
